@@ -700,6 +700,7 @@ int set_ndis_auth_mode(struct ndis_device *wnd, ULONG auth_mode)
 	case Ndis802_11AuthModeWPAPSK:
 		wnd->iw_auth_wpa_version = IW_AUTH_WPA_VERSION_WPA;
 		wnd->iw_auth_key_mgmt = IW_AUTH_KEY_MGMT_PSK;
+        break;
 	case Ndis802_11AuthModeWPANone:
 		wnd->iw_auth_wpa_version = IW_AUTH_WPA_VERSION_DISABLED;
 		wnd->iw_auth_key_mgmt = IW_AUTH_KEY_MGMT_PSK;
@@ -1074,7 +1075,7 @@ static char *ndis_translate_scan(struct net_device *dev,
 	/* add protocol name */
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = SIOCGIWNAME;
-	strncpy(iwe.u.name, network_type_to_name(bssid->net_type), IFNAMSIZ);
+	strncpy(iwe.u.name, network_type_to_name(bssid->net_type), IFNAMSIZ - 1);
 	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_CHAR_LEN);
 	if (ret == event)
 		return NULL;
