@@ -77,7 +77,6 @@ u64 wrap_ticks_to_boot;
 #if defined(CONFIG_X86_64)
 static struct timer_list shared_data_timer;
 struct kuser_shared_data kuser_shared_data;
-static void update_user_shared_data_proc(unsigned long data);
 #endif
 
 WIN_SYMBOL_MAP("KeTickCount", &jiffies)
@@ -91,7 +90,11 @@ DEFINE_PER_CPU(struct irql_info, irql_info);
 #endif
 
 #if defined(CONFIG_X86_64)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void update_user_shared_data_proc(struct timer_list *tl)
+#else
 static void update_user_shared_data_proc(unsigned long data)
+#endif
 {
 	/* timer is supposed to be scheduled every 10ms, but bigger
 	 * intervals seem to work (tried up to 50ms) */
@@ -407,9 +410,17 @@ static void initialize_object(struct dispatcher_header *dh, enum dh_type type,
 	InitializeListHead(&dh->wait_blocks);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void timer_proc(struct timer_list *tl)
+#else
 static void timer_proc(unsigned long data)
+#endif
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	struct wrap_timer *wrap_timer = from_timer(wrap_timer, tl, timer);
+#else
 	struct wrap_timer *wrap_timer = (struct wrap_timer *)data;
+#endif
 	struct nt_timer *nt_timer;
 	struct kdpc *kdpc;
 
@@ -452,9 +463,13 @@ void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
 		return;
 	}
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)
 	init_timer(&wrap_timer->timer);
-	wrap_timer->timer.data = (unsigned long)wrap_timer;
 	wrap_timer->timer.function = timer_proc;
+	wrap_timer->timer.data = (unsigned long)wrap_timer;
+#else
+	timer_setup(&wrap_timer->timer, timer_proc, 0);
+#endif
 	wrap_timer->nt_timer = nt_timer;
 #ifdef TIMER_DEBUG
 	wrap_timer->wrap_timer_magic = WRAP_TIMER_MAGIC;
@@ -1746,10 +1761,19 @@ wstdcall void __iomem *WIN_FUNC(MmMapIoSpace,3)
 {
 	void __iomem *virt;
 	ENTER1("cache type: %d", cache);
-	if (cache == MmCached)
+	if (cache == MmCached) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,6,0)
+		virt = ioremap_cache(phys_addr, size);
+#else
 		virt = ioremap(phys_addr, size);
-	else
+#endif
+	} else {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,6,0)
+		virt = ioremap(phys_addr, size);
+#else
 		virt = ioremap_nocache(phys_addr, size);
+#endif
+	}
 	TRACE1("%llx, %zu, %p", phys_addr, size, virt);
 	return virt;
 }
@@ -2485,8 +2509,6 @@ struct worker_init_struct {
 
 int ntoskernel_init(void)
 {
-	struct timeval now;
-
 	spin_lock_init(&dispatcher_lock);
 	spin_lock_init(&ntoskernel_lock);
 	spin_lock_init(&ntos_work_lock);
@@ -2505,10 +2527,23 @@ int ntoskernel_init(void)
 	INIT_WORK(&ntos_work, ntos_work_worker);
 	wrap_timer_slist.next = NULL;
 
-	do_gettimeofday(&now);
-	wrap_ticks_to_boot = TICKS_1601_TO_1970;
-	wrap_ticks_to_boot += (u64)now.tv_sec * TICKSPERSEC;
-	wrap_ticks_to_boot += now.tv_usec * 10;
+    wrap_ticks_to_boot = TICKS_1601_TO_1970;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,0,0)
+    do {
+        u64 now;
+        now = ktime_get_real_ns();
+        wrap_ticks_to_boot += now * 10;
+    } while (0);
+#else
+    do {
+        struct timeval now;
+
+        do_gettimeofday(&now);
+        wrap_ticks_to_boot += (u64)now.tv_sec * TICKSPERSEC;
+        wrap_ticks_to_boot += now.tv_usec * 10;
+    } while (0);
+#endif
+	TRACE2("%llu", wrap_ticks_to_boot);
 	wrap_ticks_to_boot -= jiffies * TICKSPERJIFFY;
 	TRACE2("%llu", wrap_ticks_to_boot);
 
@@ -2524,7 +2559,7 @@ int ntoskernel_init(void)
 			info->task = NULL;
 			info->count = 0;
 #ifdef CONFIG_SMP
-			cpumask_setall(&info->cpus_allowed);
+			cpumask_setall(tsk_cpus_allowed(info));
 #endif
 		}
 	} while (0);
@@ -2559,9 +2594,13 @@ int ntoskernel_init(void)
 #if defined(CONFIG_X86_64)
 	memset(&kuser_shared_data, 0, sizeof(kuser_shared_data));
 	*((ULONG64 *)&kuser_shared_data.system_time) = ticks_1601();
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)
 	init_timer(&shared_data_timer);
 	shared_data_timer.function = update_user_shared_data_proc;
 	shared_data_timer.data = 0;
+#else
+	timer_setup(&shared_data_timer, update_user_shared_data_proc, 0);
+#endif
 #endif
 	return 0;
 }
