@@ -457,10 +457,18 @@ static int setup_tx_sg_list(struct ndis_device *wnd, struct sk_buff *skb,
 	for (i = 0; i < skb_shinfo(skb)->nr_frags; i++) {
 		skb_frag_t *frag = &skb_shinfo(skb)->frags[i];
 		sg_element++;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,4,0)
+		sg_element->length = skb_frag_size(frag);
+#else
 		sg_element->length = frag->size;
+#endif
 		sg_element->address =
 			pci_map_page(wnd->wd->pci.pdev, skb_frag_page(frag),
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,4,0)
+				     skb_frag_off(frag), skb_frag_size(frag),
+#else
 				     frag->page_offset, frag->size,
+#endif
 				     PCI_DMA_TODEVICE);
 		TRACE3("%llx, %u", sg_element->address, sg_element->length);
 	}
@@ -704,7 +712,7 @@ static void tx_worker(struct work_struct *work)
 			n = wnd->max_tx_packets;
 		n = mp_tx_packets(wnd, wnd->tx_ring_start, n);
 		if (n) {
-			wnd->net_dev->trans_start = jiffies;
+			netif_trans_update(wnd->net_dev);
 			wnd->tx_ring_start =
 				(wnd->tx_ring_start + n) % TX_RING_SIZE;
 			wnd->is_tx_ring_full = 0;
@@ -1089,9 +1097,17 @@ send_assoc_event:
 	EXIT2(return);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void iw_stats_timer_proc(struct timer_list *tl)
+#else
 static void iw_stats_timer_proc(unsigned long data)
+#endif
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	struct ndis_device *wnd = from_timer(wnd, tl, iw_stats_timer);
+#else
 	struct ndis_device *wnd = (struct ndis_device *)data;
+#endif
 
 	ENTER2("%d", wnd->iw_stats_interval);
 	if (wnd->iw_stats_interval > 0) {
@@ -1107,8 +1123,12 @@ static void add_iw_stats_timer(struct ndis_device *wnd)
 		return;
 	if (wnd->iw_stats_interval < 0)
 		wnd->iw_stats_interval *= -1;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	timer_setup(&wnd->iw_stats_timer, iw_stats_timer_proc, 0);
+#else
 	wnd->iw_stats_timer.data = (unsigned long)wnd;
 	wnd->iw_stats_timer.function = iw_stats_timer_proc;
+#endif
 	mod_timer(&wnd->iw_stats_timer, jiffies + wnd->iw_stats_interval);
 }
 
@@ -1120,9 +1140,17 @@ static void del_iw_stats_timer(struct ndis_device *wnd)
 	EXIT2(return);
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void hangcheck_proc(struct timer_list *tl)
+#else
 static void hangcheck_proc(unsigned long data)
+#endif
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	struct ndis_device *wnd = from_timer(wnd, tl, hangcheck_timer);
+#else
 	struct ndis_device *wnd = (struct ndis_device *)data;
+#endif
 
 	ENTER3("%d", wnd->hangcheck_interval);
 	if (wnd->hangcheck_interval > 0) {
@@ -1143,8 +1171,12 @@ void hangcheck_add(struct ndis_device *wnd)
 		wnd->hangcheck_interval = hangcheck_interval * HZ;
 	if (wnd->hangcheck_interval < 0)
 		wnd->hangcheck_interval *= -1;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	timer_setup(&wnd->hangcheck_timer, hangcheck_proc, 0);
+#else
 	wnd->hangcheck_timer.data = (unsigned long)wnd;
 	wnd->hangcheck_timer.function = hangcheck_proc;
+#endif
 	mod_timer(&wnd->hangcheck_timer, jiffies + wnd->hangcheck_interval);
 	EXIT2(return);
 }
@@ -2134,9 +2166,14 @@ static NTSTATUS ndis_add_device(struct driver_object *drv_obj,
 	wnd->dma_map_count = 0;
 	wnd->dma_map_addr = NULL;
 	wnd->nick[0] = 0;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	timer_setup(&wnd->hangcheck_timer, hangcheck_proc, 0);
+	timer_setup(&wnd->iw_stats_timer, hangcheck_proc, 0);
+#else
 	init_timer(&wnd->hangcheck_timer);
-	wnd->scan_timestamp = 0;
 	init_timer(&wnd->iw_stats_timer);
+#endif
+	wnd->scan_timestamp = 0;
 	wnd->iw_stats_interval = 10 * HZ;
 	wnd->ndis_pending_work = 0;
 	memset(&wnd->essid, 0, sizeof(wnd->essid));
